@@ -6,8 +6,9 @@ use async_tungstenite::{
 };
 use futures::FutureExt;
 use robespierre_models::{
+    auth::Session,
     events::{ClientToServerEvent, ServerToClientEvent},
-    id::{ChannelId, UserId},
+    id::ChannelId,
 };
 use std::result::Result as StdResult;
 use tokio::{net::TcpStream, sync::mpsc::UnboundedSender};
@@ -44,13 +45,16 @@ pub struct Connection(ConnectionInternal);
 /// A value that can be used to authenticate on the websocket, either as a bot or as a non-bot user.
 #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub enum Authentication<'a> {
-    Bot {
-        token: &'a str,
-    },
-    User {
-        user_id: UserId,
-        session_token: &'a str,
-    },
+    Bot { token: &'a str },
+    User { session_token: &'a str },
+}
+
+impl<'a> From<&'a Session> for Authentication<'a> {
+    fn from(s: &'a Session) -> Self {
+        Self::User {
+            session_token: &s.token.0,
+        }
+    }
 }
 
 #[async_trait::async_trait]
@@ -94,8 +98,18 @@ pub trait Context: Sized + Clone + Send + 'static {
 impl Connection {
     /// Connects to the websocket, and authenticates, returning the socket or an error if it failed.
     pub async fn connect<'a>(auth: impl Into<Authentication<'a>>) -> Result<Self> {
-        tracing::debug!("Connecting to websocket");
-        let (stream, _response) = connect_async("wss://ws.revolt.chat").await?;
+        Self::connect_with_url(auth, "wss://ws.revolt.chat").await
+    }
+
+    /// Connects to the websocket on the specified url, and authenticates, returning the socket or an error if it failed.
+    ///
+    /// Use if connecting to a self-hosted instance of revolt; otherwise use [Self::connect].
+    pub async fn connect_with_url<'a>(
+        auth: impl Into<Authentication<'a>>,
+        url: &str,
+    ) -> Result<Self> {
+        tracing::debug!("Connecting to websocket on {}", url);
+        let (stream, _response) = connect_async(url).await?;
         let mut internal = ConnectionInternal {
             stream,
             closed: false,
@@ -131,7 +145,7 @@ impl Connection {
             TypingManagerTick,
         }
 
-        let mut typing_session_manager = typing::TypingSessionManager::new();
+        let mut typing_session_manager = typing::TypingSessionManager::default();
 
         loop {
             // Event::FromServer = we got an event from the server, which we should pass to the handler
@@ -228,24 +242,20 @@ impl Connection {
 
 impl ConnectionInternal {
     async fn hb(&mut self) -> Result {
-        self.send_event(ClientToServerEvent::Ping { time: 0, data: (0,) })
+        self.send_event(ClientToServerEvent::Ping { data: 0 })
             .await?;
 
         Ok(())
     }
 
-    async fn authenticate<'a>(&mut self, auth: Authentication<'a>) -> Result {
+    async fn authenticate(&mut self, auth: Authentication<'_>) -> Result {
         tracing::debug!("Authenticating");
         self.send_event(match &auth {
-            Authentication::Bot { token } => ClientToServerEvent::AuthenticateBot {
+            Authentication::Bot { token } => ClientToServerEvent::Authenticate {
                 token: token.to_string(),
             },
-            Authentication::User {
-                user_id,
-                session_token,
-            } => ClientToServerEvent::Authenticate {
-                user_id: user_id.clone(),
-                session_token: session_token.to_string(),
+            Authentication::User { session_token } => ClientToServerEvent::Authenticate {
+                token: session_token.to_string(),
             },
         })
         .await?;
@@ -274,9 +284,7 @@ impl ConnectionInternal {
 
         tracing::debug!("[>] {}", &json);
 
-        self.stream
-            .send(TungsteniteMessage::text(json))
-            .await?;
+        self.stream.send(TungsteniteMessage::text(json)).await?;
 
         Ok(())
     }
